package dev.kske.chess.game;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import dev.kske.chess.board.Move;
import dev.kske.chess.board.Piece.Color;
import dev.kske.chess.uci.UCIHandle;
import dev.kske.chess.uci.UCIListener;
import dev.kske.chess.uci.UCIOption;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>UCIPlayer.java</strong><br>
 * Created: <strong>18.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class UCIPlayer extends Player implements UCIListener {

	private UCIHandle handle;

	private String			name, author;
	private List<UCIOption>	options;

	public UCIPlayer(Color color, String enginePath) {
		super(color);
		options = new ArrayList<>();
		try {
			handle = new UCIHandle(enginePath);
			handle.setListener(this);
			handle.start();
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	@Override
	public void requestMove() {
		handle.positionFEN(board.toFEN());
		handle.go();
	}

	@Override
	public void cancelMove() {
		handle.stop();
	}

	@Override
	public void disconnect() {
		handle.quit();
	}

	@Override
	public void onIdName(String name) {
		this.name = name;
	}

	@Override
	public void onIdAuthor(String author) {
		this.author = author;
	}

	@Override
	public void onUCIOk() {
		System.out.println("UCI ok");
	}

	@Override
	public void onReadyOk() {
		System.out.println("Ready ok");
	}

	@Override
	public void onBestMove(String move) {
		Move moveObj = Move.fromAlgebraicNotation(move);
		game.onMove(this, moveObj);
	}

	@Override
	public void onBestMove(String move, String ponderMove) {
		onBestMove(move);
	}

	@Override
	public void onCopyProtectionChecking() {
		System.out.println("Copy protection checking...");
	}

	@Override
	public void onCopyProtectionOk() {
		System.out.println("Copy protection ok");
	}

	@Override
	public void onCopyProtectionError() {
		System.err.println("Copy protection error!");
	}

	@Override
	public void onRegistrationChecking() {
		System.out.println("Registration checking...");
	}

	@Override
	public void onRegistrationOk() {
		System.out.println("Registration ok");
	}

	@Override
	public void onRegistrationError() {
		System.err.println("Registration error!");
	}

	@Override
	public void onInfo(Map<String, String> additionalInfo) {
		System.out.println("Info:");
		additionalInfo.forEach((k, v) -> System.out.printf("%s: %s%n", k, v));
	}

	@Override
	public void onOption(UCIOption option) {
		options.add(option);
	}
}
