package dev.kske.chess.uci;

import java.util.Map;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>UCIListener.java</strong><br>
 * Created: <strong>19.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public interface UCIListener {

	/**
	 * Identifies the name of the engine.
	 * 
	 * @param name The name of the engine
	 */
	void onIdName(String name);

	/**
	 * Identifies the author of the engine.
	 * 
	 * @param author The name of the engine's author
	 */
	void onIdAuthor(String author);

	/**
	 * The engine is ready in UCI mode.
	 */
	void onUCIOk();

	/**
	 * The engine has processed all inputs and is ready for new commands.
	 */
	void onReadyOk();

	/**
	 * The engine has stopped searching and has found the best move.
	 * 
	 * @param move The best moves the engine has found
	 */
	void onBestMove(String move);

	/**
	 * The engine has stopped searching and has found the best move.
	 * 
	 * @param move       The best move the engine has found
	 * @param ponderMove The move the engine likes to ponder on
	 */
	void onBestMove(String move, String ponderMove);

	/**
	 * The engine will check the copy protection now.
	 */
	void onCopyProtectionChecking();

	/**
	 * The engine has successfully checked the copy protection.
	 */
	void onCopyProtectionOk();

	/**
	 * The engine has encountered an error during copy protection checking.
	 */
	void onCopyProtectionError();

	/**
	 * The engine will check the registration now.
	 */
	void onRegistrationChecking();

	/**
	 * The engine has successfully checked the registration.
	 */
	void onRegistrationOk();

	/**
	 * The engine has encountered an error during registration checking.
	 */
	void onRegistrationError();

	/**
	 * The engine sends information to the GUI.
	 * 
	 * @param additionalInfo Contains all pieces of information to be sent
	 */
	void onInfo(Map<String, String> additionalInfo);

	/**
	 * Tells the GUI which parameters can be changed in the engine.
	 * 
	 * @param option Option object describing the parameter
	 */
	void onOption(UCIOption option);
}
